@@ -155,26 +155,6 @@ gnome_two_password_dialog_finalize (GObject *object)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
-static gboolean
-is_visible (GtkWidget *w)
-{
-#if GTK_CHECK_VERSION(2,20,0)
-	return gtk_widget_get_visible (w);
-#else
-	return GTK_WIDGET_VISIBLE (w);
-#endif
-}
-
-static gboolean
-is_sensitive (GtkWidget *w)
-{
-#if GTK_CHECK_VERSION(2,18,0)
-	return gtk_widget_get_sensitive (w);
-#else
-	return GTK_WIDGET_SENSITIVE (w);
-#endif
-}
-
 /* GtkDialog callbacks */
 static void
 dialog_show_callback (GtkWidget *widget, gpointer callback_data)
@@ -183,15 +163,15 @@ dialog_show_callback (GtkWidget *widget, gpointer callback_data)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (callback_data);
 
-	if (   is_visible (password_dialog->details->username_entry)
-	    && !password_dialog->details->readonly_username) {
+	if (gtk_widget_get_visible (password_dialog->details->username_entry) &&
+	    !password_dialog->details->readonly_username) {
 		gtk_widget_grab_focus (password_dialog->details->username_entry);
-	} else if (   is_visible (password_dialog->details->domain_entry)
-	           && !password_dialog->details->readonly_domain) {
+	} else if (gtk_widget_get_visible (password_dialog->details->domain_entry) &&
+		   !password_dialog->details->readonly_domain) {
 		gtk_widget_grab_focus (password_dialog->details->domain_entry);
-	} else if (is_visible (password_dialog->details->password_entry)) {
+	} else if (gtk_widget_get_visible (password_dialog->details->password_entry)) {
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	} else if (is_visible (password_dialog->details->password_entry_secondary)) {
+	} else if (gtk_widget_get_visible (password_dialog->details->password_entry_secondary)) {
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 	}
 }
@@ -285,14 +265,14 @@ username_entry_activate (GtkWidget *widget, GtkWidget *dialog)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (dialog);
 	
-	if (is_visible (password_dialog->details->domain_entry) &&
-	    is_sensitive (password_dialog->details->domain_entry))
+	if (gtk_widget_get_visible (password_dialog->details->domain_entry) &&
+	    gtk_widget_get_sensitive (password_dialog->details->domain_entry))
 		gtk_widget_grab_focus (password_dialog->details->domain_entry);
-	else if (is_visible (password_dialog->details->password_entry) &&
-		 is_sensitive (password_dialog->details->password_entry))
+	else if (gtk_widget_get_visible (password_dialog->details->password_entry) &&
+		 gtk_widget_get_sensitive (password_dialog->details->password_entry))
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	else if (is_visible (password_dialog->details->password_entry_secondary) &&
-		 is_sensitive (password_dialog->details->password_entry_secondary))
+	else if (gtk_widget_get_visible (password_dialog->details->password_entry_secondary) &&
+		 gtk_widget_get_sensitive (password_dialog->details->password_entry_secondary))
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 }
 
@@ -303,11 +283,11 @@ domain_entry_activate (GtkWidget *widget, GtkWidget *dialog)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (dialog);
 	
-	if (is_visible (password_dialog->details->password_entry) &&
-	    is_sensitive (password_dialog->details->password_entry))
+	if (gtk_widget_get_visible (password_dialog->details->password_entry) &&
+	    gtk_widget_get_sensitive (password_dialog->details->password_entry))
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	else if (is_visible (password_dialog->details->password_entry_secondary) &&
-		 is_sensitive (password_dialog->details->password_entry_secondary))
+	else if (gtk_widget_get_visible (password_dialog->details->password_entry_secondary) &&
+		 gtk_widget_get_sensitive (password_dialog->details->password_entry_secondary))
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 }
 
@@ -347,7 +327,9 @@ gnome_two_password_dialog_new (const char	*dialog_title,
 	action_area = GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog)));
 
 	/* Setup the dialog */
+#if !GTK_CHECK_VERSION (2,22,0)
 	gtk_dialog_set_has_separator (dialog, FALSE);
+#endif
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_box_set_spacing (content, 2); /* 2 * 5 + 2 = 12 */
 	gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
